import os
import json

import requests
from django.utils.translation import get_language


def translate(key: str) -> str:
    base_url = os.environ.get('TRANSLATION_SERVICE_URL')
    lang = get_language()
    if lang == 'uz-cyr':
        lang = 'crl'
    if lang not in ['uz', 'ru', 'crl']:
        lang = 'ru'
    return requests.get(f'{base_url}/api/translations/{key}', params={
        'lang': lang
    }).text


def translate_month(month: 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 | 10 | 11 | 12):
    return json.loads(translate('primeng.monthNames'))[month - 1]


def translate_weekday(weekday: 1 | 2 | 3 | 4 | 5 | 6 | 7):
    return json.loads(translate('primeng.dayNames'))[weekday % 7]



from .dict_util import get_by_dotted as dict_get_dotted
from .date import EDate, EDateTime, to_edatetime, to_edate, generate_months
from .data import EData
from .color import generate_random_color, hex_to_rgb, rgb_to_hex, darken_color
from .compute_total_and_difference import compute_total_and_difference, compute_difference_percent



from django.db import connection


def fetchone(query: str, params=None, desc=True, null=False):
    if params is None:
        params = []
    with connection.cursor() as cursor:
        cursor.execute(query, params)
        rows = cursor.fetchone()
        if not desc:
            return rows
        columns = [desc[0] for desc in cursor.description]
        if null and rows is None:
            return dict.fromkeys(columns, None)
        return dict(zip(columns, rows))

def fetchall(query: str, params=None, desc=True):
    if params is None:
        params = []
    with connection.cursor() as cursor:
        cursor.execute(query, params)
        rows = cursor.fetchall()
        if not desc:
            return rows
        columns = [desc[0] for desc in cursor.description]
        return [dict(zip(columns, row)) for row in rows]



class AOServices:
    GES = 'ges'
    IES = 'ies'
    HET = 'het'
    MET = 'MET'
    HUDUDGAZ_LIQUEFIED_GAS = 'hududgaz-liquefied-gas'
    HUDUDGAZ_NATURAL_GAS = 'hududgaz-natural-gas'
    NEFTEGAZ_LIQUID_HYDROCARBONS = 'liquid-hydrocarbons'
    UZBEKNEFTEGAZ_OIL_WELL = 'oil-well'
    UZBEKNEFTEGAZ_NATURAL_GAZ = 'uzbekneftegaz'
    UZ_GAZ_TRADE = 'uz-gas-trade'
    UZ_TRANS_GAZ = 'uz-trans-gaz'
    UZBEKKUMIR = 'uzbekkumir'
    TRANSLATION = 'translation'
    USER_MANAGEMENT = 'user-management'



import os

from rest_framework.permissions import IsAuthenticated

from energy_base.utils.microservices import is_internal_microservice_request


class BaseRolePermissions(IsAuthenticated):
    required_role = None
    role_apps = set()

    @classmethod
    def add_role_app(cls, app):
        if 'role_apps' not in cls.__dict__:
            cls.role_apps = set()
        cls.role_apps.add(app)

    def has_permission(self, request, view):
        # Check IsAuthenticated
        if not super().has_permission(request, view):
            return False

        # Give full access for superadmin or admin
        if request.user.is_superuser or 'admin' in request.user.roles:
            return True

        # Give full access for microservice
        if not os.getenv('DONT_CHECK_MICROSERVICE_REQUEST') and is_internal_microservice_request(request):
            return True

        # Check application role access
        app_name = view.__class__.__module__.split('.')[1]
        if self.role_apps and app_name not in self.role_apps:
            return False

        # Check role access
        if hasattr(request.user, 'roles'):
            return self.required_role in request.user.roles

        return False


from io import BytesIO

from openpyxl import load_workbook

from .base_sheet_processor import BaseSheetProcessor


class XlsProcessor:

    def __init__(self, file: str | bytes):
        self.file = file
        self.wb = load_workbook(BytesIO(self.file))

    def get_data(self, sheet_processor: BaseSheetProcessor):
        return sheet_processor.get_data(self.wb)

    def validate(self, sheet_processor: BaseSheetProcessor):
        return sheet_processor.validate(self.wb)

    def validate_many(self, sheet_processors: list[BaseSheetProcessor]):
        errors = {}
        for sheet_processor in sheet_processors:
            error = self.validate(sheet_processor)
            if error:
                errors.update(error)

        return errors or None



from copy import deepcopy
from typing import Any

from django.db.models import QuerySet


def deep_map(data: dict | list, func_cond, func_map, in_place=True):
    if not in_place:
        data = deepcopy(data)

    if isinstance(data, dict):
        for key, value in data.items():
            if isinstance(value, (list, dict, QuerySet)):
                deep_map(value, func_cond, func_map, True)
            elif func_cond(value):
                data[key] = func_map(value)
    elif isinstance(data, (list, QuerySet)):
        for index, value in enumerate(data):
            if isinstance(value, (list, dict, QuerySet)):
                deep_map(value, func_cond, func_map, True)
            elif func_cond(value):
                data[index] = func_map(value)

    return data


def null_to_zero(data: dict | list, in_place=True):
    return deep_map(data, lambda value: value is None, lambda _: 0, in_place)


def deep_round(data: dict | list, ndigits: int, in_place=True):
    return deep_map(data, lambda value: isinstance(value, float), lambda value: round(value, ndigits), in_place)


class EData:
    def __init__(self, data):
        self.data = data

    def null_to_zero(self):
        null_to_zero(self.data)
        return self

    def round(self, ndigits: int):
        deep_round(self.data, ndigits)
        return self

    def map(self, func_cond, func_map):
        deep_map(self.data, func_cond, func_map)
        return self


def key_mapper(obj: dict, key_dict: dict, delete_other=True):
    new_obj = {}
    for old_key, new_key in key_dict.items():
        new_obj[new_key] = obj[old_key]

    if not delete_other:
        for key, value in obj.items():
            if key not in key_dict:
                new_obj[key] = value

    return new_obj


def items_values_list(items: list[dict] | Any, *keys, flat=True):
    values_list = []
    for item in items:
        if len(keys) == 1 and flat:
            values_list.append(item[keys[0]])
        else:
            values_list.append(tuple(item[key] for key in keys))

    return values_list



from urllib.parse import urlparse

from drf_spectacular.utils import extend_schema
from rest_framework import views, status
from rest_framework.response import Response

from energy_base.external_api.serializers import ApiStatusSerializer
from energy_base.services.telnet import Telnet


@extend_schema(tags=['admin/external-api'])
class BaseApiStatusView(views.APIView):
    serializer_class = ApiStatusSerializer
    api_base_url = None

    def get(self, request):
        parsed_url = urlparse(self.api_base_url)
        host = parsed_url.hostname
        port = parsed_url.port or (443 if parsed_url.scheme == 'https' else 80)
        telnet = Telnet(host=host, port=port)
        serializer = self.serializer_class(data={
            'url': self.api_base_url,
            'host': host,
            'port': port,
            'status': telnet.status,
            'error_message': telnet.error_message
        })
        serializer.is_valid(raise_exception=True)
        return Response(data=serializer.data, status=status.HTTP_200_OK)



import uuid

from django.conf import settings
from django.db import models

from ..models import ImportFile


class TemporaryData(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    file = models.ForeignKey(ImportFile, on_delete=models.CASCADE)
    data = models.JSONField()
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    deleted_at = models.DateTimeField(blank=True, null=True)
    created_by = models.UUIDField()
    updated_by = models.UUIDField(blank=True, null=True)
    deleted_by = models.UUIDField(blank=True, null=True)

    class Meta:
        app_label = settings.FILE_UPLOAD_APP_LABEL or 'file_upload'
        db_table = 'temporary_data'
        ordering = ('-created_at',)



from rest_framework import serializers


class DashboardDateSerializer(serializers.Serializer):
    datetime = serializers.DateTimeField()
    period = serializers.ChoiceField(choices=['daily', 'hourly'], default='daily')



import telnetlib

from django.db import models


class ConnectionStatus(models.Choices):
    OK = "Ok"
    CONNECTION_REFUSED = "Connection Refused"
    TIMEOUT = "Timeout"
    ERROR = "Error"


class Telnet:
    status: ConnectionStatus = None
    error_message: str = None

    def __init__(self, host, port, timeout=5):
        try:
            with telnetlib.Telnet(host, port, timeout=timeout) as tn:
                self.status = ConnectionStatus.OK
        except ConnectionRefusedError:
            self.status = ConnectionStatus.CONNECTION_REFUSED
        except TimeoutError:
            self.status = ConnectionStatus.TIMEOUT
        except Exception as e:
            self.error_message = str(e)
            self.status = ConnectionStatus.ERROR



from django.urls import path

from energy_base.api.routers import OptionalSlashRouter

from energy_base.q_api.views import TaskViewSet, ScheduleViewSet, ScheduleRunNowView

router = OptionalSlashRouter()
router.register('tasks', TaskViewSet, basename='tasks')
router.register('schedules', ScheduleViewSet, basename='schedules')

urlpatterns = [
    path('schedules/<pk>/run-now/', ScheduleRunNowView.as_view(), name='schedules-run-now'),
]

urlpatterns += router.urls



from datetime import timedelta

from django.db.models import Manager
from drf_spectacular.utils import extend_schema
from rest_framework import generics

from .base import BaseAPIView
from .serializers import DashboardDateSerializer
from ..utils import to_edatetime, to_edate


class TranslatedListView(generics.ListAPIView, BaseAPIView):
    manager: Manager

    def get_queryset(self):
        self.update_lang()
        return self.manager.all()


@extend_schema(parameters=[DashboardDateSerializer])
class BaseDashboardIndustryView(BaseAPIView):
    serializer_class = DashboardDateSerializer

    def get_period_data(self, request, prev=False):
        serializer = self.serializer_class(data=request.query_params)
        serializer.is_valid(raise_exception=True)

        period = serializer.validated_data['period']
        datetime = serializer.validated_data['datetime']

        given_period = to_edatetime(datetime).first_second_of_hour() if period == 'hourly' else datetime.date()
        if prev:
            prev_period = (given_period - timedelta(hours=1)) if period == 'hourly' else to_edate(given_period).yesterday()
            return period, given_period, prev_period

        return period, given_period



from django.http import HttpRequest
from rest_framework.request import Request


def is_internal_microservice_request(request: HttpRequest | Request):
    # check request sending from gateway, if so it is not internal
    return 'HTTP_X_FORWARDED_PREFIX' not in request.META



from energy_base.authentications.models import MicroserviceUser
from energy_base.authentications.microservice_authentication import MicroserviceUserAuthentication
__all__ = [
    'MicroserviceUser',
    'MicroserviceUserAuthentication',
]
